﻿using System;
using System.Diagnostics.Eventing.Reader;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Xml.Serialization;
using Newtonsoft.Json;

namespace UDPBroadcasExercise
{
    internal class UDPServerService
    {
        private int port;
        private Random rand;

        public UDPServerService(int port)
        {
            this.port = port;
            rand = new Random(DateTime.Now.Millisecond);
        }

        public void Start()
        {
            Socket simpleSockect = new Socket(AddressFamily.InterNetwork, 
                SocketType.Dgram, ProtocolType.Udp);
            simpleSockect.EnableBroadcast = true;

            Socket jsonSocket = new Socket(AddressFamily.InterNetwork,
                SocketType.Dgram, ProtocolType.Udp);
            jsonSocket.EnableBroadcast = true;

            Socket xmlSocket = new Socket(AddressFamily.InterNetwork,
                SocketType.Dgram, ProtocolType.Udp);
            xmlSocket.EnableBroadcast = true;

            SensorData data;
            while (true)
            {
                data = new SensorData(rand.Next(255), rand.Next(255));

                // Simple
                String buffer = String.Format("light: {0}\r\ntemp: {1}\r\ntimestamp: {2}\r\n", 
                    data.Light, data.Temp, data.Timestamp);
                Console.WriteLine(buffer);
                byte[] sendbuf = Encoding.ASCII.GetBytes(buffer);
                IPEndPoint ep = new IPEndPoint(IPAddress.Broadcast, port);
                simpleSockect.SendTo(sendbuf, ep);
                

                // Json
                String jsonbuffer = JsonConvert.SerializeObject(data);
                Console.WriteLine(jsonbuffer);
                byte[] sendbuf2 = Encoding.ASCII.GetBytes(jsonbuffer);
                IPEndPoint ep2 = new IPEndPoint(IPAddress.Broadcast, port+1);
                simpleSockect.SendTo(sendbuf2, ep2);


                // Xml
                String xmlbuffer = SerializeObjectToXmlString(data);
                Console.WriteLine(xmlbuffer);
                byte[] sendbuf3 = Encoding.ASCII.GetBytes(xmlbuffer);
                IPEndPoint ep3 = new IPEndPoint(IPAddress.Broadcast, port + 2);
                simpleSockect.SendTo(sendbuf3, ep3);

                Thread.Sleep(1000);
            }
        }

        public static string SerializeObjectToXmlString(SensorData toSerialize)
        {
            XmlSerializer xmlSerializer = new XmlSerializer(toSerialize.GetType());

            using (StringWriter textWriter = new StringWriter())
            {
                xmlSerializer.Serialize(textWriter, toSerialize);
                return textWriter.ToString();
            }
        }
    }
}